unit StrStuff;
{
Author: Craig Manley
Date: 10 January 2000
}

interface

uses
  SysUtils, Classes, Math;

const
  crlf = #13#10;
  c1 = 52845;
  c2 = 22719;

function BoolToStr(const b: boolean): string;
function CapFirst(const s: string): string;
function CRC16(CRC: Word; Data: Pointer; DataSize: LongWord): Word; assembler;
function CRC32(CRC: LongWord; Data: Pointer; DataSize: LongWord): LongWord; assembler;
procedure DelimitedToStrings(AStr: string; AStrings: TStrings);
function Encrypt(const s: string; Key: word): string;
function Decrypt(const s: string; Key: word): string;
function HexStrToStr(const s: string): string;
function IsAlpha(const s: string): boolean;
function IsAlphaNum(const s: string): boolean;
function LastChar(const s: string): char;
function LastPos(const Substr, S: string): integer;
function Mod11Check(const Number: Cardinal): byte; overload;
function Mod11Check(const Number: string): byte; overload;
function PosFrom(const StartPos: integer; const Substr, S: string): integer;
function Rev(const s: string): string;
function StrToCardinal(const s: string): cardinal;
function StrToCardinalDef(const s: string; def: cardinal): cardinal;
function StrDef(const s, sDefault: string): string;
function StrRep(AOldStr, ANewStr, ATargetStr: string): string;
function StrToHexStr(const s: string): string;


implementation

function BoolToStr(const b: boolean): string;
begin
  if b then
    Result := 'True'
  else
    Result := 'False';
end;

function CapFirst(const s: string): string;
var
  i: integer;
begin
  i := length(s);
  if i > 0 then
    Result := AnsiUpperCase(copy(s,1,1)) + AnsiLowerCase(copy(s,2,i-1))
  else
    Result := '';
end;

function CRC16(CRC: Word; Data: Pointer; DataSize: LongWord): Word; assembler;
asm
         AND    EDX,EDX
         JZ     @Exit
         AND    ECX,ECX
         JLE    @Exit
         PUSH   EBX
         PUSH   EDI
         XOR    EBX,EBX
         LEA    EDI,CS:[OFFSET @CRC16]
@Start:  MOV    BL,[EDX]
         XOR    BL,AL
         SHR    AX,8
         XOR    AX,[EDI + EBX * 2]
         INC    EDX
         DEC    ECX
         JNZ    @Start
         POP    EDI
         POP    EBX
@Exit:   RET
         NOP
@CRC16:  DW     00000h, 0C0C1h, 0C181h, 00140h, 0C301h, 003C0h, 00280h, 0C241h
         DW     0C601h, 006C0h, 00780h, 0C741h, 00500h, 0C5C1h, 0C481h, 00440h
         DW     0CC01h, 00CC0h, 00D80h, 0CD41h, 00F00h, 0CFC1h, 0CE81h, 00E40h
         DW     00A00h, 0CAC1h, 0CB81h, 00B40h, 0C901h, 009C0h, 00880h, 0C841h
         DW     0D801h, 018C0h, 01980h, 0D941h, 01B00h, 0DBC1h, 0DA81h, 01A40h
         DW     01E00h, 0DEC1h, 0DF81h, 01F40h, 0DD01h, 01DC0h, 01C80h, 0DC41h
         DW     01400h, 0D4C1h, 0D581h, 01540h, 0D701h, 017C0h, 01680h, 0D641h
         DW     0D201h, 012C0h, 01380h, 0D341h, 01100h, 0D1C1h, 0D081h, 01040h
         DW     0F001h, 030C0h, 03180h, 0F141h, 03300h, 0F3C1h, 0F281h, 03240h
         DW     03600h, 0F6C1h, 0F781h, 03740h, 0F501h, 035C0h, 03480h, 0F441h
         DW     03C00h, 0FCC1h, 0FD81h, 03D40h, 0FF01h, 03FC0h, 03E80h, 0FE41h
         DW     0FA01h, 03AC0h, 03B80h, 0FB41h, 03900h, 0F9C1h, 0F881h, 03840h
         DW     02800h, 0E8C1h, 0E981h, 02940h, 0EB01h, 02BC0h, 02A80h, 0EA41h
         DW     0EE01h, 02EC0h, 02F80h, 0EF41h, 02D00h, 0EDC1h, 0EC81h, 02C40h
         DW     0E401h, 024C0h, 02580h, 0E541h, 02700h, 0E7C1h, 0E681h, 02640h
         DW     02200h, 0E2C1h, 0E381h, 02340h, 0E101h, 021C0h, 02080h, 0E041h
         DW     0A001h, 060C0h, 06180h, 0A141h, 06300h, 0A3C1h, 0A281h, 06240h
         DW     06600h, 0A6C1h, 0A781h, 06740h, 0A501h, 065C0h, 06480h, 0A441h
         DW     06C00h, 0ACC1h, 0AD81h, 06D40h, 0AF01h, 06FC0h, 06E80h, 0AE41h
         DW     0AA01h, 06AC0h, 06B80h, 0AB41h, 06900h, 0A9C1h, 0A881h, 06840h
         DW     07800h, 0B8C1h, 0B981h, 07940h, 0BB01h, 07BC0h, 07A80h, 0BA41h
         DW     0BE01h, 07EC0h, 07F80h, 0BF41h, 07D00h, 0BDC1h, 0BC81h, 07C40h
         DW     0B401h, 074C0h, 07580h, 0B541h, 07700h, 0B7C1h, 0B681h, 07640h
         DW     07200h, 0B2C1h, 0B381h, 07340h, 0B101h, 071C0h, 07080h, 0B041h
         DW     05000h, 090C1h, 09181h, 05140h, 09301h, 053C0h, 05280h, 09241h
         DW     09601h, 056C0h, 05780h, 09741h, 05500h, 095C1h, 09481h, 05440h
         DW     09C01h, 05CC0h, 05D80h, 09D41h, 05F00h, 09FC1h, 09E81h, 05E40h
         DW     05A00h, 09AC1h, 09B81h, 05B40h, 09901h, 059C0h, 05880h, 09841h
         DW     08801h, 048C0h, 04980h, 08941h, 04B00h, 08BC1h, 08A81h, 04A40h
         DW     04E00h, 08EC1h, 08F81h, 04F40h, 08D01h, 04DC0h, 04C80h, 08C41h
         DW     04400h, 084C1h, 08581h, 04540h, 08701h, 047C0h, 04680h, 08641h
         DW     08201h, 042C0h, 04380h, 08341h, 04100h, 081C1h, 08081h, 04040h
end;

function CRC32(CRC: LongWord; Data: Pointer; DataSize: LongWord): LongWord; assembler;
asm
         AND    EDX,EDX
         JZ     @Exit
         AND    ECX,ECX
         JLE    @Exit
         PUSH   EBX
         PUSH   EDI
         XOR    EBX,EBX
         LEA    EDI,CS:[OFFSET @CRC32]
@Start:  MOV    BL,AL
         SHR    EAX,8
         XOR    BL,[EDX]
         XOR    EAX,[EDI + EBX * 4]
         INC    EDX
         DEC    ECX
         JNZ    @Start
         POP    EDI
         POP    EBX
@Exit:   RET
         DB 0, 0, 0, 0, 0 // Align Table
@CRC32:  DD 000000000h, 077073096h, 0EE0E612Ch, 0990951BAh
         DD 0076DC419h, 0706AF48Fh, 0E963A535h, 09E6495A3h
         DD 00EDB8832h, 079DCB8A4h, 0E0D5E91Eh, 097D2D988h
         DD 009B64C2Bh, 07EB17CBDh, 0E7B82D07h, 090BF1D91h
         DD 01DB71064h, 06AB020F2h, 0F3B97148h, 084BE41DEh
         DD 01ADAD47Dh, 06DDDE4EBh, 0F4D4B551h, 083D385C7h
         DD 0136C9856h, 0646BA8C0h, 0FD62F97Ah, 08A65C9ECh
         DD 014015C4Fh, 063066CD9h, 0FA0F3D63h, 08D080DF5h
         DD 03B6E20C8h, 04C69105Eh, 0D56041E4h, 0A2677172h
         DD 03C03E4D1h, 04B04D447h, 0D20D85FDh, 0A50AB56Bh
         DD 035B5A8FAh, 042B2986Ch, 0DBBBC9D6h, 0ACBCF940h
         DD 032D86CE3h, 045DF5C75h, 0DCD60DCFh, 0ABD13D59h
         DD 026D930ACh, 051DE003Ah, 0C8D75180h, 0BFD06116h
         DD 021B4F4B5h, 056B3C423h, 0CFBA9599h, 0B8BDA50Fh
         DD 02802B89Eh, 05F058808h, 0C60CD9B2h, 0B10BE924h
         DD 02F6F7C87h, 058684C11h, 0C1611DABh, 0B6662D3Dh
         DD 076DC4190h, 001DB7106h, 098D220BCh, 0EFD5102Ah
         DD 071B18589h, 006B6B51Fh, 09FBFE4A5h, 0E8B8D433h
         DD 07807C9A2h, 00F00F934h, 09609A88Eh, 0E10E9818h
         DD 07F6A0DBBh, 0086D3D2Dh, 091646C97h, 0E6635C01h
         DD 06B6B51F4h, 01C6C6162h, 0856530D8h, 0F262004Eh
         DD 06C0695EDh, 01B01A57Bh, 08208F4C1h, 0F50FC457h
         DD 065B0D9C6h, 012B7E950h, 08BBEB8EAh, 0FCB9887Ch
         DD 062DD1DDFh, 015DA2D49h, 08CD37CF3h, 0FBD44C65h
         DD 04DB26158h, 03AB551CEh, 0A3BC0074h, 0D4BB30E2h
         DD 04ADFA541h, 03DD895D7h, 0A4D1C46Dh, 0D3D6F4FBh
         DD 04369E96Ah, 0346ED9FCh, 0AD678846h, 0DA60B8D0h
         DD 044042D73h, 033031DE5h, 0AA0A4C5Fh, 0DD0D7CC9h
         DD 05005713Ch, 0270241AAh, 0BE0B1010h, 0C90C2086h
         DD 05768B525h, 0206F85B3h, 0B966D409h, 0CE61E49Fh
         DD 05EDEF90Eh, 029D9C998h, 0B0D09822h, 0C7D7A8B4h
         DD 059B33D17h, 02EB40D81h, 0B7BD5C3Bh, 0C0BA6CADh
         DD 0EDB88320h, 09ABFB3B6h, 003B6E20Ch, 074B1D29Ah
         DD 0EAD54739h, 09DD277AFh, 004DB2615h, 073DC1683h
         DD 0E3630B12h, 094643B84h, 00D6D6A3Eh, 07A6A5AA8h
         DD 0E40ECF0Bh, 09309FF9Dh, 00A00AE27h, 07D079EB1h
         DD 0F00F9344h, 08708A3D2h, 01E01F268h, 06906C2FEh
         DD 0F762575Dh, 0806567CBh, 0196C3671h, 06E6B06E7h
         DD 0FED41B76h, 089D32BE0h, 010DA7A5Ah, 067DD4ACCh
         DD 0F9B9DF6Fh, 08EBEEFF9h, 017B7BE43h, 060B08ED5h
         DD 0D6D6A3E8h, 0A1D1937Eh, 038D8C2C4h, 04FDFF252h
         DD 0D1BB67F1h, 0A6BC5767h, 03FB506DDh, 048B2364Bh
         DD 0D80D2BDAh, 0AF0A1B4Ch, 036034AF6h, 041047A60h
         DD 0DF60EFC3h, 0A867DF55h, 0316E8EEFh, 04669BE79h
         DD 0CB61B38Ch, 0BC66831Ah, 0256FD2A0h, 05268E236h
         DD 0CC0C7795h, 0BB0B4703h, 0220216B9h, 05505262Fh
         DD 0C5BA3BBEh, 0B2BD0B28h, 02BB45A92h, 05CB36A04h
         DD 0C2D7FFA7h, 0B5D0CF31h, 02CD99E8Bh, 05BDEAE1Dh
         DD 09B64C2B0h, 0EC63F226h, 0756AA39Ch, 0026D930Ah
         DD 09C0906A9h, 0EB0E363Fh, 072076785h, 005005713h
         DD 095BF4A82h, 0E2B87A14h, 07BB12BAEh, 00CB61B38h
         DD 092D28E9Bh, 0E5D5BE0Dh, 07CDCEFB7h, 00BDBDF21h
         DD 086D3D2D4h, 0F1D4E242h, 068DDB3F8h, 01FDA836Eh
         DD 081BE16CDh, 0F6B9265Bh, 06FB077E1h, 018B74777h
         DD 088085AE6h, 0FF0F6A70h, 066063BCAh, 011010B5Ch
         DD 08F659EFFh, 0F862AE69h, 0616BFFD3h, 0166CCF45h
         DD 0A00AE278h, 0D70DD2EEh, 04E048354h, 03903B3C2h
         DD 0A7672661h, 0D06016F7h, 04969474Dh, 03E6E77DBh
         DD 0AED16A4Ah, 0D9D65ADCh, 040DF0B66h, 037D83BF0h
         DD 0A9BCAE53h, 0DEBB9EC5h, 047B2CF7Fh, 030B5FFE9h
         DD 0BDBDF21Ch, 0CABAC28Ah, 053B39330h, 024B4A3A6h
         DD 0BAD03605h, 0CDD70693h, 054DE5729h, 023D967BFh
         DD 0B3667A2Eh, 0C4614AB8h, 05D681B02h, 02A6F2B94h
         DD 0B40BBE37h, 0C30C8EA1h, 05A05DF1Bh, 02D02EF8Dh
         DD 074726F50h, 0736E6F69h, 0706F4320h, 067697279h
         DD 028207468h, 031202963h, 020393939h, 048207962h
         DD 06E656761h, 064655220h, 06E616D64h, 06FBBA36Eh
end;


procedure DelimitedToStrings(AStr: string; AStrings: TStrings);
var
  i, len: integer;
begin
  AStrings.Clear;
  len := Length(aStr);
  i := MaxIntValue([Pos(';',AStr),Pos(',',AStr)]);
  while (i > 0) do begin
    if (i > 1) then
      AStrings.Add(Copy(AStr,1,i-1));
    AStr := Copy(AStr,i+1,len);
    i := MaxIntValue([Pos(';',AStr),Pos(',',AStr)]);
  end;
  if AStr > '' then
    AStrings.Add(AStr);
end;


function Encrypt(const s: string; Key: word): string;
var
  i: byte;
begin
  SetLength(Result,Length(s));
  for i := 1 to Length(s) do
  begin
    Result[i] := char(byte(s[i]) xor (Key shr 8));
    Key := (byte(Result[i]) + Key) * c1 + c2;
  end;
end;

function Decrypt(const s: string; Key: word): string;
var
  i: byte;
begin
  SetLength(Result,Length(s));
  for i := 1 to Length(s) do
  begin
    Result[i] := char(byte(s[i]) xor (Key shr 8));
    Key := (byte(s[i]) + Key) * c1 + c2;
  end;
end;

function HexStrToStr(const s: string): string;
var
  i, len: integer;
  HexChar: string[2];
begin
  Result := '';
  len := Length(s);
  if (len = 0) or ((len mod 2) > 0) then
    Exit;
  for i := 0 to (len shr 1)-1 do
  begin
    HexChar := Copy(s,i*2+1,2);
    Result := Result + Chr(StrToInt('$'+HexChar));
  end;
end;

function IsAlpha(const s: string): boolean;
var
  i: integer;
begin
  Result := True;
  for i := 1 to Length(s) do
  begin
    if not ((s[i] in ['a'..'z']) or (s[i] in ['A'..'Z'])) then
    begin
      Result := False;
      Exit;
    end;
  end;
end;

function IsAlphaNum(const s: string): boolean;
var
  i: integer;
begin
  Result := True;
  for i := 1 to Length(s) do
  begin
    if not ((s[i] in ['a'..'z']) or (s[i] in ['A'..'Z']) or (s[i] in ['0'..'9'])) then
    begin
      Result := False;
      Exit;
    end;
  end;
end;

function LastChar(const s: string): char;
begin
  if s = '' then
    Result := char(0)
  else
    Result := s[Length(s)];
end;

function Mod11Check(const Number: Cardinal): byte;
begin
  Result := Mod11Check(IntToStr(Number));
end;

function Mod11Check(const Number: string): byte;
var
  i, len, lvnTotal: integer;
begin
  len := Length(Number);
  lvnTotal := 0;
  for i := 1 to len do
    inc(lvnTotal,StrToInt(Number[i]) * ((len+1) - i));
  Result := lvnTotal mod 11;
  if Result >= 10 then
    Dec(Result,10);
end;


function LastPos(const Substr, S: string): integer;
var
  i : integer;
begin
  Result := 0;
  if Substr = '' then
    Exit;
  Result := Pos(Substr, S);
  if Result = 0 then
    Exit;
  while Result <> Length(S) do
  begin
    i := PosFrom(Result+1,Substr,S);
    if i = 0 then
      Break;
    Result := i;
  end;
end;

function PosFrom(const StartPos: integer; const Substr, S: string): integer;
var
  fc,l,i: integer;
begin
  Result := 0;
  fc := StartPos;
  if fc <= 1 then           // start at beginning
  begin
    Result := Pos(Substr,S);
    Exit;
  end;
  l := Length(S);
  if fc > l then            // start beyond end
    Exit;
  i := Pos(Substr,copy(S,fc,l));
  if i = 0 then
    Exit;                   // not found
  Result := fc + i - 1;     // adjust for offset
end;

function Rev(const s: string): string;
var
  i : integer;
begin
  SetLength(Result,Length(s));
  for i := 1 to Length(s) do
    Result[(Length(s)-i)+1] := s[i];
end;

function StrToCardinal(const s: string): cardinal;
var
  i: Int64;
begin
  i := StrToInt64(s);
  if (i > High(Cardinal)) or (i < Low(Cardinal)) then
    raise EConvertError.Create(s + ' is not a valid Cardinal number.');
  Result := i;
end;


function StrToCardinalDef(const s: string; def: cardinal): cardinal;
var
  i: Int64;
begin
  i := StrToInt64Def(s,def);
  if (i > High(Cardinal)) or (i < Low(Cardinal)) then
    Result := 0
  else
    Result := i;
end;

function StrDef(const s, sDefault: string): string;
begin
  if s = '' then
    Result := sDefault
  else
    Result := s;
end;

function StrToHexStr(const s: string): string;
var
  i: integer;
begin
  for i := 1 to Length(s) do
  begin
    Result := Result + IntToHex(Ord(s[i]),2);
  end;
end;

function StrRep(AOldStr, ANewStr, ATargetStr: string): string;
var
  i, iLastPos: integer;
begin
  Result := ATargetStr;
  i := Pos(AOldStr ,Result);
  iLastPos := i + Length(ANewStr);
  while (i > 0) do
  begin
    Result := copy(Result,1,i-1)+ANewStr+copy(Result,i+length(AOldStr),MaxInt);
    i := pos(AOldStr,copy(Result,iLastPos,MaxInt));
    if i > 0 then
    begin
      i := i + iLastPos - 1;
      iLastPos := i + Length(ANewStr);
    end;
  end;
end;

end.
